﻿//////////////////////////////////////////////
// Buffer.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	struct MapRequestDescriptor ;
	struct ResourceCpuDataDescriptor ;
	struct ResourceGpuCopyDescriptor ;
	struct UnmapRequestDescriptor ;
}

/// Includes ---------------------------------
	
// nkGraphics
#include "MapResult.h"

#include "../Dll/DllDefines.h"

#include "../Resources/ShaderResource.h"

#include "../Utilities/DxDefinesWrapper.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Buffer : public ShaderResource, public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Data originales
			ResourceCpuDataDescriptor* _cpuData ;

			// Données
			unsigned int _elementByteSize ;
			unsigned int _elementCount ;

			// Usages actifs
			bool _computeUseActivated ;
			bool _shaderUseActivated ;
			bool _outputStreamUse ;
			
		public :
		
			// Constructeur, destructeur
			Buffer () ;
			Buffer (const Buffer& other) = delete ;
			virtual ~Buffer () ;

			// Getters
			unsigned int getElementByteSize () const ;
			unsigned int getElementCount () const ;
			unsigned int getTotalByteSize () const ;
			// Plus abstrait
			virtual USAGE getUsage () const = 0 ;
			virtual BIND_FLAG getBindFlag () const = 0 ;
			virtual CPU_ACCESS_FLAG getCpuAccessFlag () const = 0 ;
			virtual RESOURCE_MISC_FLAG getMiscFlag () const = 0 ;

			// Setters
			void setCpuData (const ResourceCpuDataDescriptor& data) ;
			virtual void freeCpuData (bool freeDataPtr) ;
			virtual void setElementByteSize (unsigned int value) ;
			virtual void setElementCount (unsigned int value) ;
			// Plus abstraits
			virtual void setUsage (USAGE value) = 0 ;
			virtual void setBindFlag (BIND_FLAG value) = 0 ;
			virtual void setCpuAccessFlag (CPU_ACCESS_FLAG value) = 0 ;
			virtual void setMiscFlag (RESOURCE_MISC_FLAG value) = 0 ;
			virtual void copyParams (const Buffer& other) = 0 ;

			// Map (Renderer)
			virtual MapResult map (const MapRequestDescriptor& requestDesc) = 0 ;
			virtual void unmap (const UnmapRequestDescriptor& requestDesc) = 0 ;
			virtual void gpuCopyFrom (Buffer& other, const ResourceGpuCopyDescriptor& descriptor) = 0 ;

			// Utilisation buffer en shader resource
			virtual void prepareForShaderResourceUsage (bool value = true) ;
			virtual void prepareForComputeResourceUsage (bool value = true) ;
			virtual void prepareForOutputStreamUsage (bool value = true) ;

			// Overload pour recopies tableau
			Buffer& operator= (const Buffer& other) = delete ;

			// Import export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}