﻿//////////////////////////////////////////////
// IVector.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMaths
{
	class Vector ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/ByteAlignedClass.h>

// Windows / dx
#include <DirectXMath.h>

/// Class ------------------------------------

namespace nkMaths
{
	class DLL_MATHS_EXPORT IVector : public ByteAlignedClass<16>
	{
		friend class Vector ;

		protected :
		
			// Attributs
			DirectX::XMVECTORI32 _values ;
			
		public :
		
			// Constructeur, destructeur
			IVector () ;
			IVector (int x, int y) ;
			IVector (int x, int y, int z) ;
			IVector (int x, int y, int z, int w) ;
			IVector (const Vector& other) ;
			IVector (const IVector& other) ;
			IVector (const DirectX::XMVECTOR& value) ;
			IVector (const DirectX::XMVECTORI32& value) ;
			~IVector () ;

			// Getters
			int getX () const ;
			int getY () const ;
			int getZ () const ;
			int getW () const ;
			DirectX::XMVECTORI32 getValues () const ;

			// Setters
			void setX (int value) ;
			void setY (int value) ;
			void setZ (int value) ;
			void setW (int value) ;

			// Ajout à une composante
			void addX (int value) ;
			void addY (int value) ;
			void addZ (int value) ;
			void addW (int value) ;
			void multX (int value) ;
			void multY (int value) ;
			void multZ (int value) ;
			void multW (int value) ;

			// Distances
			float getLength () const ;
			int getLengthSquared () const ;
			int getDistanceSquared (const IVector& other) const ;
			float getDistance (const IVector& other) const ;

			// Pratique
			void fromString (const char* str) ;

			// Opérateurs de base
			// Egalité
			IVector& operator= (const IVector& other) ;
			IVector& operator= (const Vector& other) ;
			// Addition
			IVector operator+ (const IVector& other) const ;
			void operator+= (const IVector& other) ;
			// Soustraction
			IVector operator- (const IVector& other) const ;
			void operator-= (const IVector& other) ;
			// Multiplication
			IVector operator* (const IVector& other) const ;
			void operator*= (const IVector& other) ;
			IVector operator* (int coeff) const ;
			void operator*= (int coeff) ;
			// Division
			IVector operator/ (const IVector& other) const ;
			void operator/= (const IVector& other) ;
			IVector operator/ (int coeff) const ;
			void operator/= (int coeff) ;
			// Comparaison
			bool operator== (const IVector& other) const ;
			bool operator< (const IVector& other) const ;
			bool operator<= (const IVector& other) const ;
			bool operator> (const IVector& other) const ;
			bool operator>= (const IVector& other) const ;
	} ;
}