﻿//////////////////////////////////////////////
// String.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMemory
{
	class StringView ;
}

/// Includes ---------------------------------

// nkMemory
#include "../Dll/DllDefines.h"

// Standards
#include <string>
#include <string_view>

/// Class ------------------------------------

namespace nkMemory
{
	class DLL_MEMORY_EXPORT String final
	{
		private :

			// Attributes
			// Data info
			char* _data ;
			unsigned long long _size ;

		public :

			// Constructor, destructor
			String () ;
			String (const char* data) ;
			String (unsigned long long size) ;
			String (const char* data, unsigned long long size) ;
			String (const StringView& view) ;
			String (const String& other) ;
			String (String&& other) noexcept ;
			~String () ;

			// Getters
			char* getData () const ;
			unsigned long long getSize () const ;
			bool empty () const ;
			char& front () ;
			char& back () ;

			// Management
			void clear () ;
			void resize (unsigned long long size) ;

			// Operators
			String& operator= (const char* data) ;
			String& operator= (const StringView& other) ;
			String& operator= (const String& other) ;
			String& operator= (String&& other) noexcept ;
			const char operator[] (unsigned long long index) const ;
			char& operator[] (unsigned long long index) ;
			void operator+= (char value) ;
			void operator+= (const StringView& other) ;
			nkMemory::String operator+ (const StringView& other) ;
			bool operator== (const char* other) ;
			bool operator== (const String& other) ;
			bool operator== (const StringView& other) ;

		public :

			// Inlined constructors
			String (const std::string& str)
			:	_data (new char [str.size() + 1]),
				_size (str.size())
			{
				// Copy memory over
				memcpy(_data, &str[0], _size) ;
				_data[_size] = '\0' ;
			}

			String (const std::string_view& str)
			:	_data (new char [str.size() + 1]),
				_size (str.size())
			{
				// Copy memory over
				memcpy(_data, &str[0], _size) ;
				_data[_size] = '\0' ;
			}

		public :

			// Inlined operators
			operator std::string () const
			{
				return std::string(_data, _size) ;
			}

			operator std::string_view () const
			{
				return std::string_view(_data, _size) ;
			}
	} ;
}