﻿//////////////////////////////////////////////
// StringView.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkMemory
{
	class String ;
}

/// Includes ---------------------------------

// nkMemory
#include "../Dll/DllDefines.h"

// Standards
#include <string>
#include <string_view>

/// Class ------------------------------------

namespace nkMemory
{
	class DLL_MEMORY_EXPORT StringView final
	{
		private :

			// Attributes
			// Data info
			const char* _data ;
			unsigned long long _size ;

		public :

			// Constructor, destructor
			StringView () ;
			StringView (const char* data) ;
			StringView (const char* data, unsigned long long size) ;
			StringView (const String& other) ;
			~StringView () ;

			// Getters
			const char* getData () const ;
			unsigned long long getSize () const ;
			bool empty () const ;
			char front () const ;
			char back () const ;

			// Operators
			StringView& operator= (const char* data) ;
			StringView& operator= (const String& other) ;
			char operator[] (unsigned long long index) const ;
			bool operator== (const char* other) ;
			bool operator== (const String& other) ;
			bool operator== (const StringView& other) ;

		public :

			// Inlined constructors
			StringView (const std::string& str)
			:	_data (&str[0]),
				_size (str.size())
			{
				// Nothing to do
			}

			StringView (const std::string& str, unsigned long long index, unsigned long long size = 0ull)
			:	_data (&str[index]),
				_size (size ? size : str.size() - index)
			{
				// Nothing to do
			}

			StringView (const std::string_view& str)
			:	_data (&str[0]),
				_size (str.size())
			{
				// Nothing to do
			}

			StringView (const std::string_view& str, unsigned long long index, unsigned long long size = 0ull)
			:	_data (&str[index]),
				_size (size ? size : str.size() - index)
			{
				// Nothing to do
			}

		public :

			// Inlined operators
			operator std::string () const
			{
				return std::string(_data, _size) ;
			}

			operator std::string_view () const
			{
				return std::string_view(_data, _size) ;
			}
	} ;
}