﻿//////////////////////////////////////////////
// EnvironmentStrategy.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class Environment ;
	class Function ;
	class Namespace ;
	class Script ;
	class ScriptObjectReference ;
	class ScriptStrategy ;
	class UserType ;
}

/// Includes ---------------------------------

// nkScripts
#include "../Functions/FunctionDefines.h"

#include "../ExecutionResult.h"

// nkMemory
#include <NilkinsMemory/Containers/Buffer.h>
#include <NilkinsMemory/Containers/BufferView.h>

/// Class ------------------------------------

namespace nkScripts
{
	class EnvironmentStrategy
	{
		protected :

			// Attributes
			// Parent environment
			Environment* _parentEnv ;
			// Erreur potentielles
			ExecutionResult _lastExecutionResult ;

		protected :

			// Functions
			// Getters
			ScriptStrategy* _getStrategyFrom (const Script* script) ;

		public :

			// Constructeur, destructeur
			EnvironmentStrategy (Environment* parentEnv) ;
			virtual ~EnvironmentStrategy () ;

			// Getters
			Environment* getParentEnv () const ;
			const ExecutionResult& getLastExecutionResult () const ;

			// Exécution
			virtual bool execute (const Script& script) = 0 ;
			virtual void requestGarbageCollection () = 0 ;
			virtual void shutdown () = 0 ;

			// Changement des vars
			virtual void setVar (const char* name, bool value) = 0 ;
			virtual void setVar (const char* name, int value) = 0 ;
			virtual void setVar (const char* name, float value) = 0 ;
			virtual void setVar (const char* name, double value) = 0 ;
			virtual void setVar (const char* name, const char* value) = 0 ;
			virtual void setVar (const char* name, const ScriptObjectReference& value) = 0 ;
			virtual void setObject (const char* name, const char* userTypeName, void* value) = 0 ;

			virtual bool isSet (const char* name) = 0 ;

			virtual bool getVar (const char* name, bool defaultValue) = 0 ;
			virtual int getVar (const char* name, int defaultValue) = 0 ;
			virtual float getVar (const char* name, float defaultValue) = 0 ;
			virtual double getVar (const char* name, double defaultValue) = 0 ;
			virtual const char* getVar (const char* name, const char* defaultValue) = 0 ;
			virtual ScriptObjectReference getVar (const char* name) = 0 ;
			virtual void* getObject (const char* name, const char* userTypeName) = 0 ;
			virtual ScriptObjectReference getScriptFunction (const char* name) = 0 ;

			// Fonctions
			virtual Function* setFunc (const char* name) = 0 ;

			// UserTypes
			virtual UserType* setUserType (const char* name) = 0 ;

			// Namespaces
			virtual Namespace* setNamespace (const char* name) = 0 ;

			// Call C++ -> Script
			virtual bool callScriptFunction (const ScriptObjectReference& reference, const DataStack& args, DataStack& expectedOutput) = 0 ;
			virtual nkMemory::Buffer serializeScriptObject (const ScriptObjectReference& reference, bool forwardUserDataOwnership) = 0 ;
			virtual ScriptObjectReference deserializeScriptObject (const nkMemory::BufferView<unsigned char>& object) = 0 ;
	} ;
}
